@@ -4,9 +4,10 @@
 #include <gd.h>
 #include <gdfonts.h>
 #include <KayaAPI.h>
+#include <stdfuns.h>
 
 extern "C" {
-    void* createImage(int x, int y);
+    void* createImage(int x, int y, int tc);
     void destroyImage(void* img);
 
     /// Return Images directly
@@ -31,6 +32,9 @@ extern "C" {
 
     void imageWritePNG(void* img, FILE* out);
     void imageWriteJPEG(void* img, FILE* out, int quality);
+    KayaValue imageGetPNG(void* img);
+    KayaValue imageGetJPEG(void* img, int quality);
+    KayaValue imageGetBinary(void* bindata, int binsize);
 
     void* fontGetSmall();
     void* fontGetLarge();
@@ -38,7 +42,7 @@ extern "C" {
     void* fontGetGiant();
     void* fontGetTiny();
 
-    void imageString(void* img, void* font, int x, int y, char* str, 
+    void imageString(void* img, void* font, int x, int y, wchar_t* str, 
 		     int col, int up);
 }
 
