@@ -3,9 +3,14 @@
 
 // Glue functions for talking to libpq
 
-#include <Heap.h>
-#include <Array.h>
+#include <stdfuns.h>
+#include <KayaAPI.h>
 #include <libpq-fe.h>
+#include <map>
+
+using namespace std;
+
+typedef enum { DBTEXT, DBINT, DBFLOAT, DBBOOL, DBTIME } DBtype;
 
 /// These class names are rubbish and confusing!!!!1!!!
 class PGCon : public gc
@@ -13,12 +18,13 @@ class PGCon : public gc
 public:
     PGconn* con;
     int ok;
+    map<int, DBtype> typids;
 };
 
 class PGRes : public gc
 {
 public:
-    Array* res_table;
+    KayaArray res_table;
     int rows;
     int cols;
     PGresult* pgres;
@@ -26,17 +32,17 @@ public:
 
 extern "C" {
 /// Returns a PGCon
-    void* pg_connect(char* conninfo);
+    void* pg_connect(wchar_t* conninfo);
 /// Tests if a connection is ok
     bool pg_ok(void* conn);
 /// Returns an error message
-    char* pg_getError(void* conn);
+    wchar_t* pg_getError(void* conn);
 /// Returns a PGRes
-    void* pg_exec(void* vmptr,void* conn,char* query);
+    void* pg_exec(void* vmptr,void* conn,wchar_t* query);
 /// Returns a column name
-    char* pg_columnname(void* vmptr,void* res, int col);
+    wchar_t* pg_columnname(void* vmptr,void* res, int col);
 /// Gets the strings from a PGRes
-    Array* pg_getstrs(void* res);
+    KayaArray pg_getstrs(void* res);
 /// Return the number of rows in a result
     int pg_numrows(void* res);
 /// Return the number of columns in a result
