@@ -9,9 +9,16 @@
     Public Licence. See COPYING for licence.
 */
 
+using namespace std;
+
+#ifndef WIN32
 #include <pthread.h>
 #define GC_PTHREADS
+#else
+#define GC_WIN32_THREADS
+#endif
 #include <gc/gc_cpp.h>
+#include <map>
 
 class Value;
 
@@ -32,13 +39,17 @@ public:
     void push_back(Value* v);
     Value* lookup(int i);
     inline int size() { return m_size; }
-    Array* copy();
+    inline int reserved_size() { return m_alloc; }
+    Array* copy(map<Value*,Value*>& done);
     void resize(int i);
     Value* shift();
 
     /// Make sure there's enough space for at least i things.
     void expectedSize(int i);
 
+    int memusage();
+//    void append(Array* xs);
+
     bool eq(Array* x);
     int cmp(Array* x);
 
