@@ -14,24 +14,32 @@ using namespace std;
 #include "VMState.h"
 #include "stdfuns.h"
 
-Closure::Closure(VMState* vm, func fn, int args, bool stack):m_fn(fn)
+Closure::Closure(VMState* vm, func fn, int args, int arity, bool stack):m_fn(fn)
 {
 //    cout << "Making closure " << (void*)(m_fn) << endl;
-    m_args=(Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*args);
+    m_args=(Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*arity);
     m_numargs=args;
     if (stack) {
 	for(int i=0;i<args;i++) {
 //	cout << "Arg" << endl;
 /// Create a *new* reference, or things break in confusing ways.
-	    Value* top = vm->topItem();
-	    m_args[i]=new Value(top->getRaw(),
-			    top->getFunTable());
-	    vm->doPop();
+	  m_args[i]=vm->doPop()->clone();
 //	m_args[i]=vm->doPop();
 	}
     }
 }
 
+void Closure::addArgs(VMState* vm, int added)
+{
+    int newnum = m_numargs+added;
+    int i;
+
+    for(i=m_numargs; i<newnum; i++) {
+        m_args[i]=vm->doPop()->clone();
+    }
+    m_numargs = newnum;
+}
+
 Closure::~Closure()
 {
     GC_FREE(m_args);
@@ -39,7 +47,6 @@ Closure::~Closure()
 
 void Closure::run(VMState* vm)
 {
-//    cout << "Running closure " << (void*)(m_fn) << " args " << m_numargs << endl;
     for(int i=m_numargs;i>0;i--) {
 	vm->push(m_args[i-1]);
     }
@@ -53,13 +60,13 @@ bool Closure::eq(Closure* c)
 
     for(int i=0;i<m_numargs;i++)
     {
-	if (!funtable_eq(m_args[i],c->m_args[i])) { return false; }
+	if (!funtable_eq(NULL,m_args[i],c->m_args[i])) { return false; }
     }
     return true;
 
 }
 
-Closure* Closure::copy()
+Closure* Closure::copy(map<Value*, Value*>& done)
 {
     // FIXME: This ought to clone the arguments to be a proper copy
     return this;
