@@ -9,16 +9,22 @@
     Public Licence. See COPYING for licence.
 */
 
+#ifndef WIN32
 #include <pthread.h>
 #define GC_PTHREADS
+#else
+#define GC_WIN32_THREADS
+#endif
 #include <gc/gc_cpp.h>
 #include "VMState.h"
 #include "Heap.h"
+#include <map>
 
 class Closure : public gc_cleanup {
 public:
-    // Create a closure from a function, which will have <args> as its locals.
-    Closure(VMState* vm, func fn,int args, bool stack=true);
+    // Create a closure from a function, which will have <args> as its locals,
+    // and space for <arity> arguments.
+    Closure(VMState* vm, func fn,int args, int arity, bool stack=true);
     ~Closure();
 
     // Run the closure. Pushes m_args onto the stack, then runs the function
@@ -31,9 +37,10 @@ public:
     Value** getArgs() { return m_args; }
 
     void setArg(int i,Value* v) { m_args[i]=v; }
+    void addArgs(VMState* vm, int i);
 
     bool eq(Closure* c);
-    Closure* copy();
+    Closure* copy(map<Value*, Value*>& done);
 
 private:
     func m_fn;
