@@ -11,12 +11,25 @@
 
 using namespace std;
 
+#ifndef WIN32
+#define SWPRINTF swprintf
+#else
+#define SWPRINTF snwprintf
+#endif
+#ifndef NOCHECK
+#define GC_DEBUG
+#endif
+#ifndef WIN32
 #include <pthread.h>
 #define GC_PTHREADS
+#else
+#define GC_WIN32_THREADS
+#endif
 #include <gc/gc_cpp.h>
 
-#include <gc/new_gc_alloc.h>
+//#include <gc/gc_allocator.h>
 #include <vector>
+#include "sizes.h"
 #include "ValueFuns.h"
 
 class VMState;
@@ -38,21 +51,27 @@ public:
     double number;
 };
 
-class Value;
+// cached values
+extern Value *zero, *one, *minusone;
+
+#define U_TAG(x) ((x->tag_arity) >> 16)
+#define U_ARITY(x) ((x->tag_arity) & 65535)
+#define THIS_TAG (tag_arity >> 16)
+#define THIS_ARITY (tag_arity & 65535)
 
 class Union : public gc_cleanup
 {
 public:
     // If getargs is true, get arguments from the stack in vm.
     Union(VMState* vm, int tag,int arity, bool getargs = true);
-    ~Union() { GC_FREE(args); }
+    ~Union() { if (args != NULL) { GC_FREE(args); } }
 
     bool eq(Union* x);
     int cmp(Union* x);
-    Union* copy();
+    int memusage();
+    Union* copy(map<Value*,Value*>& done);
 
-    int tag;
-    int arity;
+    int tag_arity; // tag in top 16 bits, arity in bottom
     Value** args;
 };
 
@@ -60,14 +79,30 @@ class Value : public gc
 {
 public:
     /// Create an initialised value.
-    Value(void* ptr, FunTable* ft);
+    Value(void* ptr, valtype ft):m_val(ptr),m_funtable(ft) {}
 
-    valtype getType() { return m_funtable->getType(); }
+    valtype getType() { return m_funtable; }
+
+  // returns a new value pointing to the same thing
+    Value* clone() {
+      return new Value(m_val,m_funtable);
+    }
 
     /// Get an integer out. Assumes you've checked/know.
-    int getInt() { return (int)m_val; };
+    kint getInt() { return (kint)m_val; };
     /// Get a real number out
-    double getReal() { return ((Real*)m_val)->number; }
+    double getReal() { 
+// FIXME: I don't know if turning this check off is 'safe'. It doesn't
+// throw an exception after all. Maybe it should?
+#ifndef NOCHECK
+	if (m_funtable==KVT_REAL)
+	    return ((Real*)m_val)->number;
+	else
+	    return 0.0; // Not too much of an overhead here.
+#else
+	    return ((Real*)m_val)->number;
+#endif	
+    }
 
     /// Get the function pointer. Assumes you've checked/know.
     Closure* getFunc() { return (Closure*)m_val; }
@@ -84,6 +119,9 @@ public:
     /// Run the closure stored here.
     void runClosure(VMState *vm);
 
+    /// Add more arguments from the stack to the closure stored here.
+    void addClosureArgs(VMState *vm, int i);
+
     /// Coercions
     void int2str();
     void str2int();
@@ -96,7 +134,7 @@ public:
     void real2str();
 
     /// Get the tag out of a union
-    int getTag() { return ((Union*)m_val)->tag; }
+    int getTag() { return U_TAG(((Union*)m_val)); }
 
 
     /// Get the raw pointer out.
@@ -104,7 +142,17 @@ public:
 
 
     /// Update to be an integer.
-    void setInt(int i);
+    void setInt(int i)
+    {
+	m_val=(void*)i;
+	m_funtable = KVT_INT;
+    }
+
+    /// Update to be an integer and return a pointer
+    Value* setIntReturn(int i) {
+	setInt(i);
+	return this;
+    }
 
     /// Update to be a real
     void setReal(double i);
@@ -112,10 +160,13 @@ public:
     /// Update to be a function.
     void setFunc(Closure* f);
 
-    /// Update to be a string.
-    void setString(String* s);
-
     /// Update to be ana rray.
+    void setString(String* str)
+    {
+	m_val=(void*)str;
+	m_funtable = KVT_STRING;
+    }
+
     void setArray(Array* a);
 
     void readString();
@@ -124,24 +175,29 @@ public:
     /// Set a raw pointer.
     void setPtr(Value* p) {
 //    cout << "Making " << this << " to " << p->m_val << endl;
+//	memcpy(this,p,sizeof(Value*));
 	m_val=p->m_val;
 	m_funtable=p->m_funtable;
     }
 
+    void addInt(kint x) {
+        m_val = (void*)((kint)m_val+x);
+    }
+
     void addVal(Value* v) {
-	m_val=(void*)((int)m_val+(int)(v->m_val));
+	m_val=(void*)((kint)m_val+(kint)(v->m_val));
     }
 
     void subVal(Value* v) {
-	m_val=(void*)((int)m_val-(int)(v->m_val));
+	m_val=(void*)((kint)m_val-(kint)(v->m_val));
     }
 
     void mulVal(Value* v) {
-	m_val=(void*)((int)m_val*(int)(v->m_val));
+	m_val=(void*)((kint)m_val*(kint)(v->m_val));
     }
 
     void divVal(Value* v) {
-	m_val=(void*)((int)m_val/(int)(v->m_val));
+	m_val=(void*)((kint)m_val/(kint)(v->m_val));
     }
 
     /// Set an array element.
@@ -149,6 +205,14 @@ public:
 
     /// Set a value from the projection of the top stack item
     void project(VMState* vm, int i);
+    void fastproject(Value* topitem, int i) {
+	Union* top=(Union*)(topitem->m_val);
+	Value* arg=top->args[i];
+	m_val=arg->m_val;
+	m_funtable = arg->m_funtable;
+    }
+
+    void fastproject(VMState* vm, int i);
     /// Project an argument out directly.
 //    void projarg(int i, int t);
 
@@ -161,34 +225,49 @@ public:
     int length();
 
     /// Get the function table out
-    FunTable* getFunTable() { return m_funtable; }
+    valtype getFunTable() { return m_funtable; }
     
 private:
     void* m_val;
-    FunTable* m_funtable;
+    valtype m_funtable;
 };
 
-class String : public gc_cleanup
+class String : public gc
 {
 public:
-    String(char* val);
-    String(char val);
+    String(wchar_t* val);
+    String(wchar_t val);
+    String(int len);
 
-    char* getVal();
+    wchar_t* getVal() { return m_str; }
 
-    int length();
-    char getIndex(int i);
-    void setIndex(int i,char c);
+    int length() { return wcslen(m_str); }
+    wchar_t getIndex(int i) { return m_str[i]; }
+    void setIndex(int i,wchar_t c) { m_str[i]=c; }
     void append(String* s);
+    void append(wchar_t c);
+    void append(wchar_t* s);
     bool eq(String* s);
+    bool eq(wchar_t* s);
     int cmp(String* s);
     int hash();
 
+  // debug/marshalling purposes
+    int space() { return m_alloc; }
+
 private:
-    char* m_str;
+    wchar_t* m_str;
     int m_alloc;
 };
 
+struct CallStackEntry : public gc {
+    wchar_t* fn_name;
+    wchar_t* file;
+    int line;
+    wchar_t* call_file;
+    int call_line;
+};
+
 class Exception : public gc
 {
 public:
@@ -196,9 +275,17 @@ public:
     void show();
     bool eq(Exception* x);
     int cmp(Exception* x);
+
+    void dumpBacktrace();
     
     String* err;
     int code;
+
+    wchar_t* throwfile;
+    int throwline;
+
+    // Need a copy of entire backtrace. Ick.
+    vector<CallStackEntry> backtrace;
 };
 
 #endif
