@@ -12,8 +12,8 @@ extern "C" {
     void poke(void* block,int offset,int val);
     int peek(void* block,int offset);
 
-    void pokestring(void* block, int offset, char* str);
-    char* peekstring(void* block, int offset);
+    void pokestring(void* block, int offset, wchar_t* str);
+    wchar_t* peekstring(void* block, int offset);
 
     void* copyBlock(void* block,int size);
     void* copyChunk(void* block,int offset,int size);
@@ -24,8 +24,8 @@ extern "C" {
     int writeBlock(FILE* f,void* block, int size);
     void* readBlock(FILE* f, KayaValue size);
 
-    char* b64binary(void* block, int len);
-    void* b64binarydec(char* block, KayaValue len);
+    wchar_t* b64binary(void* block, int len);
+    void* b64binarydec(wchar_t* block, KayaValue len);
 }
 
 #endif
