@@ -13,19 +13,19 @@ public:
     bool matched;
     Array* matches;
 
-    char* before;
-    char* after;
+    wchar_t* before;
+    wchar_t* after;
 };
 
 extern "C" {
 
-void* re_compile(char* pattern);
-void* re_match(void* pc, char* str);
+void* re_compile(wchar_t* pattern, kint flagcode);
+void* re_match(void* pc, wchar_t* str);
 
 Array* getmatches(void* mo);
 bool matched(void* mo);
-char* getBefore(void* mo);
-char* getAfter(void* mo);
+wchar_t* getBefore(void* mo);
+wchar_t* getAfter(void* mo);
 
 }
 
